//
//  CoraoolLib.h
//  Coraool
//
//  Created by 果 on 2024/4/16.
//

#import <Foundation/Foundation.h>
#import "CoraoolRequest.h"
#import "CoraoolResponse.h"
#import <WebKit/WKWebView.h>
#import <WebKit/WebKit.h>
#import "CoraoolJsBridge.h"

/// <#Description#>
@interface CoraoolLib : NSObject


/// 获取实例
+(CoraoolLib *)getInstance;


/**
 UserTracker
 UserTracker :获取deviceId
 */

/// UserTracker
/// UserTracker :获取deviceId
+(NSString *) deviceId;

/// 【必须】设置appKey
///
/// 使用例子：
/// [[CoraoolLib getInstance] setAppKey:@"appKey"];
///
/// @param appKey 设置的appKey
-(void)setAppKey:(NSString *)appKey;


/// 【必须】设置app 安全appSec
///
/// 使用例子：
/// [[CoraoolLib getInstance] setAppSec:@"appSec"];
///
/// @param appSec UserTracker :设置appSec
-(void)setAppSec:(NSString *)appSec;



/// 【必须】设置app 的appId
///
/// 使用例子：
/// [[CoraoolLib getInstance] setAppId:@"c18"];
///
/// @param appId :设置app 的appId
-(void)setAppId:(NSString *)appId;

/// 设置app 的userId
///
/// 使用例子：
/// [[CoraoolLib getInstance] setUserId:@"1234"];
///
/// @param userId :设置app 的登录用户的userId
-(void)setUserId:(NSString *)userId;

/// 获取appKey
-(NSString *)getAppKey;

/// 获取appSec
-(NSString *)getAppSec;

/// 获取appId
-(NSString *)getAppId;

/// 获取userId
-(NSString *)getUserId;

/// UserTracker :发送日志
/// @param eventData  发送的数据，字典类型
-(void)track:(NSDictionary *)eventData;

/// UserTracker :页面埋点
/// page 埋点，trackPageStart 和trackPageEnd 成对出现，建议在viewDidAppear
/// 设计思路：
/// 在start的时候，启动以页面对象为key的数据
/// 在end的时候，会上传日志信息
///
/// 使用例子：
/// [[CoraoolLib getInstance] trackPageStart:self pageName:@"FirstViewController"];
///
/// @param pageObject 当前页面实例
/// @param pageName 页面名称
-(void)trackPageStart:(id)pageObject pageName:(NSString *)pageName;

/// UserTracker :页面埋点
/// 更新pageName，需要在trackPageStart和trackPageEnd之间
///
/// 使用例子：
/// [[CoraoolLib getInstance] updatePageName:self pageName:@"firstPage"];
///
/// @param pageObject 当前页面实例
/// @param pageName 页面名称
-(void)updatePageName:(id)pageObject pageName:(NSString *)pageName;

/// UserTracker :页面埋点
/// 更新业务参数，需要在trackPageStart和trackPageEnd之间
///
/// 使用例子：
/// NSMutableDictionary *eventValue = [NSMutableDictionary new];
/// [eventValue setValue:@"updatePagePropVal1" forKey:@"updatePagePropKey1"];
/// [[CoraoolLib getInstance] updatePageProperties:self properties:eventValue];
///
///
/// @param pageObject 当前页面实例
/// @param properties 业务参数
-(void)updatePageProperties:(id)pageObject properties:(NSDictionary *)properties;

/**
UserTracker :页面埋点
page 埋点，start 和end 成对出现，建议在viewDidDisappear
 */


/// UserTracker :页面结束埋点，会传参数
///
/// 使用例子：
/// [[CoraoolLib getInstance] trackPageEnd:self];
///
/// @param pageObject 当前页面实例
-(void)trackPageEnd:(id)pageObject;

/**
 Invoke
 1.构造请求体
 2.同步请求，异步请求
 3.游戏内置
 */

/// 同步请求
/// @param modelClass 返回后的类型
/// @param request 请求参数
-(id )syncInvoke:(Class)modelClass
         request:(CoraoolRequest *)request;


/// 异步请求
/// @param modelClass 返回后的类型
/// @param request 请求参数
/// @param success 成功后的block
/// @param failed 失败后的block
/// @param systemError 系统错误的block
-(void )asyncInvoke:(Class)modelClass
            request:(CoraoolRequest *)request
            success:(void (^)(CoraoolResponse *response))success
             failed:(void (^)(CoraoolResponse *response))failed
        systemError:(void (^)(CoraoolResponse *response))systemError;


/// 设置游戏数据
///@param dictionary 游戏数据
///@param api 请求api

-(void)setCache:(NSDictionary *)dictionary api:(NSString *)api;


/// 适合不更改WKWebView基类的方式
/// @param wkWebView wkWebView
/// @param target 代理方法的实现类
-(void)addJSBridge:(WKWebView *)wkWebView target:(id)target;

/// 适合不更改WKWebView基类的方式
/// @param wkWebView wkWebView
/// @param target 代理方法的实现类
/// @param userContentController userContentController
/// @param message 前端返回的消息
-(void)evaluateJS:(WKWebView *)wkWebView target:(id)target userContentController:(WKUserContentController *)userContentController didReceiveScriptMessage:(WKScriptMessage *)message;
@end
